/* Arbore Binar de Cautare
 * Binary Search Tree
 * 
 * Operatii: O(log2(n))
*/
#include <iostream>
#include <fstream>

template <typename T>
class Bst {
private:
	// Tip Membru (Inner Type)
	struct Node {
		T key;
		Node *ls, *rs;   // left-son, right-son
	};
	
	// Date Membre
	Node* root;
	std::ofstream fout;
	
	// Functii Membre
	
	void Insert(Node* p, const T& key)
	{
		if (p == nullptr)
		{
			root = new Node {key, nullptr, nullptr};
			return;
		}
		
		if (key <= p->key)
		{
		
			if (p->ls)
				Insert(p->ls, key);
			else
				p->ls = new Node {key, nullptr, nullptr};
		}
		else
		{
			if (p->rs)
				Insert(p->rs, key);
			else
				p->rs = new Node {key, nullptr, nullptr};
		}
	}
	
	// Parcurgeri
	
	void InOrder(Node* p) // In Ordine
	{
		if (p == nullptr)
			return;
		InOrder(p->ls);
		fout << p->key << ' ';
		InOrder(p->rs);
	}
	
	void PreOrder(Node* p)
	{
		if (p == nullptr)
			return;
		fout << p->key << ' ';
		PreOrder(p->ls);
		PreOrder(p->rs);
	}
	
	void PostOrder(Node* p)
	{
		if (p == nullptr)
			return;
		PostOrder(p->ls);
		PostOrder(p->rs);
		fout << p->key << ' ';
	}
	
	T MinValue(Node* p)
	{
		while (p && p->ls)
			p = p->ls;
		return p->key;
	}
	
public:
	Bst(const char* fileName = nullptr) :
		root {nullptr}
	{
		if (fileName != nullptr)
			fout.open(fileName);
	}
	
	
	Bst(const Bst&) = delete; //nu maI ARE default Copy-Constructor
	Bst& operator = (const Bst&) = delete; // nu mai este atribuibil
	
	void Insert(const T& key)
	{
		Insert(root, key);
	}
	
	void InOrder()
	{
		fout << "InOrder: ";
		InOrder(root);
		fout << '\n';
	}
	
	void PreOrder()
	{
		fout << "PreOrder: ";
		PreOrder(root);
		fout << '\n';
	}
	
	void PostOrder()
	{
		fout << "PostOrder: ";
		PostOrder(root);
		fout << '\n';
	}
	
	T MinValue()
	{
		return MinValue(root);
	}
	
};

int main()
{
	Bst<int> bst("arb.out");
	
	Bst<int> bst2;
//	bst = bst2;
	std::ifstream fin("arb.in");
	
	int x;
	while (fin >> x)
		bst.Insert(x);
	
	bst.InOrder();
	bst.PreOrder();
	bst.PostOrder();
	
	std::cout << bst.MinValue();
}
